/*
 * Bundesliga is Copyright (c) 1997-1999 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#include "bl.h"
#include "blrcp.h"

/**
 * Database Handling Routinen
 */

static DmOpenRef BlDB;
static SelectedDBInfo dbInfo[10];
CharPtr Items[10] = { 0, };
static Err err = 0;

static Short FindInstalledDatabases();
static Short DBNameNr(CharPtr);

DmOpenRef CreateDatabase(CharPtr name, Prefs* dbPrefs)
{
  UInt      index;
  VoidHand  RecHandle;
  DmOpenRef db;
  LocalID   dbID;

  if (DBNameNr(name)) {
    FrmAlert(ID_FrmDBExists);
    err = -1;
    return NULL;
  }

  if ( (err = DmCreateDatabase(0, name, 'tpBL', 'Data', false)) )
    return NULL;
  dbID = DmFindDatabase(0, name);
  db = DmOpenDatabase(0, dbID, dmModeReadWrite);

  index = 0;
  RecHandle = DmNewRecord(db, &index, sizeof(Prefs));
  DmWrite(MemHandleLock(RecHandle), 0, dbPrefs, sizeof(Prefs));
  MemHandleUnlock(RecHandle);
  DmReleaseRecord(db, index, true);

  index = 1;
  RecHandle = DmNewRecord(db, &index, 1520);
  DmSet(MemHandleLock(RecHandle), 0, 1520, 255);
  MemHandleUnlock(RecHandle); 
  DmReleaseRecord(db, index, true);

  index = 2;
  RecHandle = DmNewRecord(db, &index, 
			  sizeof(programme)+sizeof(team)+sizeof(Datum));
  DmReleaseRecord(db, index, true);
  return db;
}

Err InitDatabase(DmOpenRef db, Boolean onlyJustCreated)
{
  VoidHand      RecHandle;
  VoidPtr	RecPointer;
  int		i, RecLen;

  RecHandle = DmGetRecord(db, 0);
  RecPointer = MemHandleLock(RecHandle);
  MemMove(&prefs, RecPointer, sizeof(prefs));
  MemHandleUnlock(RecHandle);
  DmReleaseRecord(db, 0, true);
  
  if (prefs.Version<3) {
    FrmAlert(ID_FrmTooOld);
    err = -1;
    return -1;
  }

  RecLen=76;
  RecHandle = DmGetRecord(db, 1);
  RecPointer = MemHandleLock(RecHandle);
  for (i=0; i<prefs.NumTeams; i++) {
    MemMove(MStatus[i].Erg, RecPointer+i*RecLen, RecLen);
  }
  MemHandleUnlock(RecHandle);
  DmReleaseRecord(db, 1, true);
  
  RecLen=sizeof(programme);
  if (!onlyJustCreated) {
    RecHandle = DmGetRecord(db, 2);
    RecPointer = MemHandleLock(RecHandle);
    MemMove(programme, RecPointer, RecLen);
    for (i=0; i<prefs.NumTeams; i++) {
      MemMove(team[i], RecPointer+RecLen+sizeof(team[0])*i, sizeof(team[0]));
    }
    MemMove(Datum, RecPointer+sizeof(programme)+sizeof(team), sizeof(Datum));
    MemHandleUnlock(RecHandle);
    DmReleaseRecord(db, 2, true);
  }

  return 0;
}

Err OpenDatabase(UInt cardNo, LocalID dbID)
{
  Boolean onlyJustCreated = false;
   
  if (dbID==0) {
    BlDB = DmOpenDatabaseByTypeCreator('Data', 'tpBL', dmModeReadWrite);
  } else {
    BlDB = DmOpenDatabase(cardNo, dbID, dmModeReadWrite);
  }

  if (!BlDB) {
    if (!(BlDB = CreateDatabase("Bundesliga 98/99", &prefs))) {
      /* CreateDatabase sets err; err==-1 => our own error */
      return err;
    }
    onlyJustCreated = true;
  }

  if (InitDatabase(BlDB, onlyJustCreated)) {
    return err;
  }

  if (!prefs.Spieltage)
    prefs.Spieltage = 2*(prefs.NumTeams + (prefs.NumTeams%2) -1);

  BerechneTabelle(prefs.Spieltage-1);
  return 0;
}

void SaveStatus(Short nr)
{
  Int i, size;
  VoidPtr p; 
  
  prefs.matchDay = matchDay;
  
  p = MemHandleLock(DmResizeRecord(BlDB, 0, sizeof(prefs)));
  DmWrite(p, 0, &prefs, sizeof(prefs));
  MemPtrUnlock(p);
  DmReleaseRecord(BlDB, 0, true);

  p = MemHandleLock(DmResizeRecord(BlDB, 1, 20*76));
  for (i=0; i<prefs.NumTeams; i++) {
    DmWrite(p, i*76, MStatus[i].Erg, 76);
  }
  MemPtrUnlock(p);
  DmReleaseRecord(BlDB, 1, true);

  size = sizeof(programme)+sizeof(team)+sizeof(Datum);
  if (!DmQueryRecord(BlDB, 2)) {
    UInt index = 2;
    VoidHand RecHandle = DmNewRecord(BlDB, &index, size);
    DmSet(MemHandleLock(RecHandle), 0, i, 255);
    MemHandleUnlock(RecHandle); 
    DmReleaseRecord(BlDB, index, true);
  }

  p = MemHandleLock(DmResizeRecord(BlDB, 2, size));
  DmWrite(p, 0, programme, sizeof(programme));
  DmWrite(p, sizeof(programme), team, sizeof(team));
  DmWrite(p, sizeof(programme)+sizeof(team), Datum, sizeof(Datum));
  MemPtrUnlock(p);
  DmReleaseRecord(BlDB, 2, true);

  dbInfo[nr].dbAttr |= dmHdrAttrBackup;
  DmSetDatabaseInfo (0, dbInfo[nr].dbID, NULL, &dbInfo[nr].dbAttr, NULL, NULL, NULL,
                     NULL, NULL, NULL, NULL, NULL, NULL);
  DmCloseDatabase(BlDB);
}

void InitNewDatabase(DmOpenRef db)
{
  Int i, size;
  VoidPtr p; 
  Word prg[380];
  char t[20][18];
  
  for (i=0; i<380; i++) {
    prg[i] = ((i%10)*2+1)*100 + ((i%10)*2+2);
  }
  for (i=0; i<20; i++) {
    StrIToA( t[i], i+1 );
  }

  size = sizeof(prg)+sizeof(t)+sizeof(Datum);
  p = MemHandleLock(DmResizeRecord(db, 2, size));
  DmWrite(p, 0, prg, sizeof(prg));
  DmWrite(p, sizeof(prg), t, sizeof(t));
  DmSet(p, sizeof(prg)+sizeof(t), sizeof(Datum), 0);
  MemPtrUnlock(p);
  DmReleaseRecord(db, 2, true);
}

static Short FindInstalledDatabases()
{
  Boolean newSearch = true;
  DmSearchStateType stateInfo;
  Short i = 0;

  while (DmGetNextDatabaseByTypeCreator(newSearch, &stateInfo, 
	 'Data', 'tpBL', false, &dbInfo[i].cardNo, &dbInfo[i].dbID) == 0 
	 && i<10) {
    newSearch = false;
    i++;
  }
  return i;
}  

static Short DBNameNr(CharPtr name)
{
  Short i;
  for (i=0; Items[i]; i++) {
    if (!StrCompare(name,Items[i]))
      return i+1;
  }
  return 0;
}

void ChooseDB()
{
  Short nrOfItems, i;
  ListPtr p;
    
  nrOfItems = FindInstalledDatabases();
  for (i=0; i<10; i++) {
    if (i<nrOfItems) {
      DmDatabaseInfo(dbInfo[i].cardNo, dbInfo[i].dbID, dbInfo[i].dbName, 
		   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
      Items[i] = dbInfo[i].dbName;
    } else {
      Items[i] = NULL;
    }
  }
  p = FrmGetListPtr(ID_FrmBl, ID_FrmPopup+1);
  LstSetListChoices(p, Items, nrOfItems);
  LstSetHeight(p, nrOfItems);
}

void SelectedDB(Short i, SelectedDBInfo *info)
{
  MemMove(info, &dbInfo[i], sizeof(SelectedDBInfo));
}

void DeleteDatabase(Short i)
{
  Err err;
  DmCloseDatabase(BlDB);
  if ((err = DmDeleteDatabase(dbInfo[i].cardNo, dbInfo[i].dbID))) {
    FrmAlert(ID_FrmDBLoeschenError);
    return;
  }
  ChooseDB();
}
